/****************************************************************************
 *
 * brute_force_matching.c
 *
 * This file is part of DDM
 *
 * Copyright (C) 2013, 2016 Moreno Marzolla
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "brute_force_matching.h"
#include "interval.h"

/**
 * Brute force approach: test each update against each subscription.
 * Returns the number of (upd, subs) matches.
 */
size_t brute_force_matching( const struct interval* sub, size_t n,
                             const struct interval* upd, size_t m )
{
    size_t result = 0;
    size_t i, j;

#pragma omp parallel for default(none) shared(n, m, sub, upd) private(i, j) reduction(+:result) schedule(dynamic,1024)
    for (i=0; i<n; ++i) {
	for (j=0; j<m; ++j) {
	    if ( intersect( &sub[i], &upd[j] ) ) {
		result++;
	    }
	}
    }

    return result;
}
