/****************************************************************************
 *
 * interval_tree_matching.c
 *
 * This file is part of DDM
 *
 * Copyright (C) 2013, 2016 Moreno Marzolla
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 ****************************************************************************/

#include "interval_tree_matching.h"
#include "int_tree.h"

size_t interval_tree_matching( const struct interval* sub, size_t n,
                               const struct interval* upd, size_t m )
{
    size_t result = 0;
    size_t i, j;
    struct int_tree tree;
    int_tree_init( &tree );
    for (i=0; i<n; ++i) {
	int_tree_insert( &tree, &sub[i] );
    }

#pragma omp parallel for default(none) shared(i, tree, n, m, sub, upd) private(j) reduction(+:result)
    for ( j=0; j<m; ++j) {
	result += int_tree_find_intersect( &tree, &upd[j], NULL, NULL );
    }
    return result;
}
